"""
Prototype class
In Development

Class representing the Station information that can be used to convert Station into another format.
"""

# from typing import List
#
# import numpy as np
#
# import redvox
# from redvox.common.session_model import SessionModel
# from redvox.common.date_time_utils import datetime_from_epoch_microseconds_utc
#
#
# class StationModel:
#     """
#     Model for a station.  Does not include data.
#
#     Properties:
#         id: string, the id of the station.  Users can change this
#
#         uuid: string, the uuid of the station.  This is set by the app when it is installed on a device
#
#         audio_sample_rate_hz: int, the audio sample rate in hz of the station
#
#         sessions: List of SessionModel: the sessions used to create the StationModel
#
#         app: string, name of the app
#
#         app_version: string, version of the app
#
#         api: float, the api version of the data
#
#         sub_api: float, the sub api of the data
#
#         make: string, make of the station
#
#         model: string, model of the station
#
#         num_sessions: int, number of sessions in the model
#
#         sdk_version: string, version of SDK used to create the model
#
#         start_date: float, timestamp of first station data point in microseconds since epoch UTC
#
#         end_date: float, timestamp of last station data point in microseconds since epoch UTC
#
#         changelog: List of str, list of all changes to station during lifetime
#     """
#     def __init__(self,
#                  station_id: str = "",
#                  uuid: str = "",
#                  app_name: str = "Redvox",
#                  api: float = np.nan,
#                  sub_api: float = np.nan,
#                  make: str = "",
#                  model: str = "",
#                  app_version: str = "",
#                  start_date: float = np.nan,
#                  end_date: float = np.nan
#                  ):
#         self.id: str = station_id
#         self.uuid: str = uuid
#         self.app_name: str = app_name
#         self.api: float = api
#         self.sub_api: float = sub_api
#         self.make: str = make
#         self.model: str = model
#         self.app_version: str = app_version
#         self.num_sessions: int = 0
#         self.sdk_version: str = redvox.version()
#         self.start_date: float = start_date
#         self.end_date: float = end_date
#         self.changelog: List[str] = []
#
#     def add_session(self, session: SessionModel):
#         """
#         Add a new session to the StationModel.  id and uuid must match; no change if no match
#
#         :param session: Session to add
#         """
#         if self.id == session.id and self.uuid == session.uuid:
#             self.num_sessions += 1
#             if self.start_date > session.start_date:
#                 self.start_date = session.start_date
#             if self.end_date < session.last_data_timestamp:
#                 self.end_date = session.last_data_timestamp
#             first_timestamp = np.nan if np.isnan(session.first_data_timestamp) \
#                 else datetime_from_epoch_microseconds_utc(
#                 session.first_data_timestamp).strftime('%Y-%m-%dT%H:%M:%S.%fZ')
#             last_timestamp = np.nan if np.isnan(session.last_data_timestamp) \
#                 else datetime_from_epoch_microseconds_utc(
#                 session.last_data_timestamp).strftime('%Y-%m-%dT%H:%M:%S.%fZ')
#             self.changelog.append(f"session from {first_timestamp}: {last_timestamp}")
#
#     @staticmethod
#     def create_from_session(session: SessionModel) -> "StationModel":
#         """
#         Create StationModel from a single session
#
#         :param session: SessionModel to create StationModel from
#         :return: StationModel
#         """
#         try:
#             result = StationModel(session.id, session.uuid, session.app_name, session.api, session.sub_api,
#                                   session.make, session.model, session.app_version, session.start_date,
#                                   session.last_data_timestamp
#                                   )
#         except Exception as e:
#             raise e
#         return result
#
#     @staticmethod
#     def create_from_sessions(sessions: List[SessionModel]) -> "StationModel":
#         """
#         create StationModel from a list of sessions.  Uses the first SessionModel as the base for the StationModel,
#         and will ignore any SessionModel that does not align with the first SessionModel
#
#         :param sessions: sessions to create StationModel from
#         :return: StationModel
#         """
#         p1 = sessions.pop(0)
#         model = StationModel.create_from_session(p1)
#         for p in sessions:
#             model.add_session(p)
#         sessions.insert(0, p1)
#         return model




SUB_API: float = 10.0  # redvox-api-1000 -> build_protos.sh -> insert_sub_api.py on 2022-07-07 00:07:12.686182

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/redvox_api_m/redvox_api_m.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#src/redvox_api_m/redvox_api_m.proto\x12\x0credvox_api_m\"\xecs\n\rRedvoxPacketM\x12\x0b\n\x03\x61pi\x18\x01 \x01(\x02\x12\x0f\n\x07sub_api\x18\x02 \x01(\x02\x12K\n\x13station_information\x18\x03 \x01(\x0b\x32..redvox_api_m.RedvoxPacketM.StationInformation\x12I\n\x12timing_information\x18\x04 \x01(\x0b\x32-.redvox_api_m.RedvoxPacketM.TimingInformation\x12\x34\n\x07sensors\x18\x05 \x01(\x0b\x32#.redvox_api_m.RedvoxPacketM.Sensors\x12>\n\revent_streams\x18\x06 \x03(\x0b\x32\'.redvox_api_m.RedvoxPacketM.EventStream\x12;\n\x08metadata\x18\x07 \x03(\x0b\x32).redvox_api_m.RedvoxPacketM.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd1$\n\x12StationInformation\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04uuid\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0f\n\x07\x61uth_id\x18\x04 \x01(\t\x12\x0c\n\x04make\x18\x05 \x01(\t\x12\r\n\x05model\x18\x06 \x01(\t\x12\x41\n\x02os\x18\x07 \x01(\x0e\x32\x35.redvox_api_m.RedvoxPacketM.StationInformation.OsType\x12\x12\n\nos_version\x18\x08 \x01(\t\x12\x13\n\x0b\x61pp_version\x18\t \x01(\t\x12\x12\n\nis_private\x18\n \x01(\x08\x12P\n\x0c\x61pp_settings\x18\x0b \x01(\x0b\x32:.redvox_api_m.RedvoxPacketM.StationInformation.AppSettings\x12V\n\x0fstation_metrics\x18\x0c \x01(\x0b\x32=.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics\x12P\n\x0cservice_urls\x18\r \x01(\x0b\x32:.redvox_api_m.RedvoxPacketM.StationInformation.ServiceUrls\x12N\n\x08metadata\x18\x0e \x03(\x0b\x32<.redvox_api_m.RedvoxPacketM.StationInformation.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xe1\x01\n\x0bServiceUrls\x12\x13\n\x0b\x61uth_server\x18\x01 \x01(\t\x12\x14\n\x0csynch_server\x18\x02 \x01(\t\x12\x1a\n\x12\x61\x63quisition_server\x18\x03 \x01(\t\x12Z\n\x08metadata\x18\x04 \x03(\x0b\x32H.redvox_api_m.RedvoxPacketM.StationInformation.ServiceUrls.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xf9\x0c\n\x0eStationMetrics\x12=\n\ntimestamps\x18\x01 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12_\n\x0cnetwork_type\x18\x02 \x03(\x0e\x32I.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics.NetworkType\x12j\n\x12\x63\x65ll_service_state\x18\x03 \x03(\x0e\x32N.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics.CellServiceState\x12\x43\n\x10network_strength\x18\x04 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12>\n\x0btemperature\x18\x05 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12:\n\x07\x62\x61ttery\x18\x06 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12\x42\n\x0f\x62\x61ttery_current\x18\x07 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12@\n\ravailable_ram\x18\x08 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12\x41\n\x0e\x61vailable_disk\x18\t \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12\x42\n\x0f\x63pu_utilization\x18\n \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12]\n\x0bpower_state\x18\x0b \x03(\x0e\x32H.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics.PowerState\x12\x62\n\x0ewifi_wake_lock\x18\x0c \x03(\x0e\x32J.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics.WifiWakeLock\x12_\n\x0cscreen_state\x18\r \x03(\x0e\x32I.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics.ScreenState\x12\x44\n\x11screen_brightness\x18\x0e \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12]\n\x08metadata\x18\x0f \x03(\x0b\x32K.redvox_api_m.RedvoxPacketM.StationInformation.StationMetrics.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"U\n\x0bNetworkType\x12\x13\n\x0fUNKNOWN_NETWORK\x10\x00\x12\x0e\n\nNO_NETWORK\x10\x01\x12\x08\n\x04WIFI\x10\x02\x12\x0c\n\x08\x43\x45LLULAR\x10\x03\x12\t\n\x05WIRED\x10\x04\"C\n\x0cWifiWakeLock\x12\x08\n\x04NONE\x10\x00\x12\r\n\tHIGH_PERF\x10\x01\x12\x0f\n\x0bLOW_LATENCY\x10\x02\x12\t\n\x05OTHER\x10\x03\"^\n\x10\x43\x65llServiceState\x12\x0b\n\x07UNKNOWN\x10\x00\x12\r\n\tEMERGENCY\x10\x01\x12\x0b\n\x07NOMINAL\x10\x02\x12\x12\n\x0eOUT_OF_SERVICE\x10\x03\x12\r\n\tPOWER_OFF\x10\x04\"O\n\nPowerState\x12\x17\n\x13UNKNOWN_POWER_STATE\x10\x00\x12\r\n\tUNPLUGGED\x10\x01\x12\x0c\n\x08\x43HARGING\x10\x02\x12\x0b\n\x07\x43HARGED\x10\x03\"F\n\x0bScreenState\x12\x18\n\x14UNKNOWN_SCREEN_STATE\x10\x00\x12\x06\n\x02ON\x10\x01\x12\x07\n\x03OFF\x10\x02\x12\x0c\n\x08HEADLESS\x10\x03\x1a\xe9\x0f\n\x0b\x41ppSettings\x12i\n\x13\x61udio_sampling_rate\x18\x01 \x01(\x0e\x32L.redvox_api_m.RedvoxPacketM.StationInformation.AppSettings.AudioSamplingRate\x12\x1a\n\x12samples_per_window\x18\x02 \x01(\x02\x12i\n\x13\x61udio_source_tuning\x18\x03 \x01(\x0e\x32L.redvox_api_m.RedvoxPacketM.StationInformation.AppSettings.AudioSourceTuning\x12h\n\x18\x61\x64\x64itional_input_sensors\x18\x04 \x03(\x0e\x32\x46.redvox_api_m.RedvoxPacketM.StationInformation.AppSettings.InputSensor\x12\x1c\n\x14\x61utomatically_record\x18\x05 \x01(\x08\x12\x1a\n\x12launch_at_power_up\x18\x06 \x01(\x08\x12\x12\n\nstation_id\x18\x07 \x01(\t\x12\x1b\n\x13station_description\x18\x08 \x01(\t\x12\x16\n\x0epush_to_server\x18\t \x01(\x08\x12\x1f\n\x17publish_data_as_private\x18\n \x01(\x08\x12\x1b\n\x13scramble_audio_data\x18\x0b \x01(\x08\x12\x18\n\x10provide_backfill\x18\x0c \x01(\x08\x12\x1f\n\x17remove_sensor_dc_offset\x18\r \x01(\x08\x12Z\n\x0b\x66\x66t_overlap\x18\x0e \x01(\x0e\x32\x45.redvox_api_m.RedvoxPacketM.StationInformation.AppSettings.FftOverlap\x12#\n\x1buse_custom_time_sync_server\x18\x0f \x01(\x08\x12\x1c\n\x14time_sync_server_url\x18\x10 \x01(\t\x12\x1e\n\x16use_custom_data_server\x18\x11 \x01(\x08\x12\x17\n\x0f\x64\x61ta_server_url\x18\x12 \x01(\t\x12\x1e\n\x16use_custom_auth_server\x18\x13 \x01(\x08\x12\x17\n\x0f\x61uth_server_url\x18\x14 \x01(\t\x12\x1e\n\x16\x61uto_delete_data_files\x18\x15 \x01(\x08\x12\x1f\n\x17storage_space_allowance\x18\x16 \x01(\x02\x12$\n\x1cuse_sd_card_for_data_storage\x18\x17 \x01(\x08\x12\x1d\n\x15use_location_services\x18\x18 \x01(\x08\x12\x14\n\x0cuse_latitude\x18\x19 \x01(\x01\x12\x15\n\ruse_longitude\x18\x1a \x01(\x01\x12\x14\n\x0cuse_altitude\x18\x1b \x01(\x02\x12P\n\x0cmetrics_rate\x18\x1c \x01(\x0e\x32:.redvox_api_m.RedvoxPacketM.StationInformation.MetricsRate\x12Z\n\x08metadata\x18\x1d \x03(\x0b\x32H.redvox_api_m.RedvoxPacketM.StationInformation.AppSettings.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"I\n\nFftOverlap\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0e\n\nPERCENT_25\x10\x01\x12\x0e\n\nPERCENT_50\x10\x02\x12\x0e\n\nPERCENT_75\x10\x03\"n\n\x11\x41udioSamplingRate\x12\x19\n\x15UNKNOWN_SAMPLING_RATE\x10\x00\x12\t\n\x05HZ_80\x10\x01\x12\n\n\x06HZ_800\x10\x02\x12\x0b\n\x07HZ_8000\x10\x03\x12\x0c\n\x08HZ_16000\x10\x04\x12\x0c\n\x08HZ_48000\x10\x05\"f\n\x11\x41udioSourceTuning\x12\x12\n\x0eUNKNOWN_TUNING\x10\x00\x12\x15\n\x11INFRASOUND_TUNING\x10\x01\x12\x14\n\x10LOW_AUDIO_TUNING\x10\x02\x12\x10\n\x0c\x41UDIO_TUNING\x10\x03\"\xa5\x03\n\x0bInputSensor\x12\x12\n\x0eUNKNOWN_SENSOR\x10\x00\x12\x11\n\rACCELEROMETER\x10\x01\x12\x16\n\x12\x41\x43\x43\x45LEROMETER_FAST\x10\x02\x12\x17\n\x13\x41MBIENT_TEMPERATURE\x10\x03\x12\t\n\x05\x41UDIO\x10\x04\x12\x14\n\x10\x43OMPRESSED_AUDIO\x10\x05\x12\x0b\n\x07GRAVITY\x10\x06\x12\r\n\tGYROSCOPE\x10\x07\x12\x12\n\x0eGYROSCOPE_FAST\x10\x08\x12\x14\n\x10IMAGE_PER_SECOND\x10\t\x12\x14\n\x10IMAGE_PER_PACKET\x10\n\x12\t\n\x05LIGHT\x10\x0b\x12\x17\n\x13LINEAR_ACCELERATION\x10\x0c\x12\x0c\n\x08LOCATION\x10\r\x12\x10\n\x0cMAGNETOMETER\x10\x0e\x12\x15\n\x11MAGNETOMETER_FAST\x10\x0f\x12\x0f\n\x0bORIENTATION\x10\x10\x12\x0c\n\x08PRESSURE\x10\x11\x12\r\n\tPROXIMITY\x10\x12\x12\x15\n\x11RELATIVE_HUMIDITY\x10\x13\x12\x13\n\x0fROTATION_VECTOR\x10\x14\x12\x0c\n\x08VELOCITY\x10\x15\"O\n\x06OsType\x12\x0e\n\nUNKNOWN_OS\x10\x00\x12\x0b\n\x07\x41NDROID\x10\x01\x12\x07\n\x03IOS\x10\x02\x12\x07\n\x03OSX\x10\x03\x12\t\n\x05LINUX\x10\x04\x12\x0b\n\x07WINDOWS\x10\x05\"D\n\x0bMetricsRate\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x13\n\x0fONCE_PER_SECOND\x10\x01\x12\x13\n\x0fONCE_PER_PACKET\x10\x02\x1a\xc0\x07\n\x11TimingInformation\x12!\n\x19packet_start_os_timestamp\x18\x01 \x01(\x01\x12#\n\x1bpacket_start_mach_timestamp\x18\x02 \x01(\x01\x12\x1f\n\x17packet_end_os_timestamp\x18\x03 \x01(\x01\x12!\n\x19packet_end_mach_timestamp\x18\x04 \x01(\x01\x12,\n$server_acquisition_arrival_timestamp\x18\x05 \x01(\x01\x12 \n\x18\x61pp_start_mach_timestamp\x18\x06 \x01(\x01\x12T\n\x0fsynch_exchanges\x18\x07 \x03(\x0b\x32;.redvox_api_m.RedvoxPacketM.TimingInformation.SynchExchange\x12\x14\n\x0c\x62\x65st_latency\x18\x08 \x01(\x02\x12\x13\n\x0b\x62\x65st_offset\x18\t \x01(\x02\x12\r\n\x05score\x18\n \x01(\x02\x12U\n\x0cscore_method\x18\x0b \x01(\x0e\x32?.redvox_api_m.RedvoxPacketM.TimingInformation.TimingScoreMethod\x12.\n\x04unit\x18\x0c \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12M\n\x08metadata\x18\r \x03(\x0b\x32;.redvox_api_m.RedvoxPacketM.TimingInformation.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x95\x02\n\rSynchExchange\x12\n\n\x02\x61\x31\x18\x01 \x01(\x01\x12\n\n\x02\x61\x32\x18\x02 \x01(\x01\x12\n\n\x02\x61\x33\x18\x03 \x01(\x01\x12\n\n\x02\x62\x31\x18\x04 \x01(\x01\x12\n\n\x02\x62\x32\x18\x05 \x01(\x01\x12\n\n\x02\x62\x33\x18\x06 \x01(\x01\x12.\n\x04unit\x18\x07 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12[\n\x08metadata\x18\x08 \x03(\x0b\x32I.redvox_api_m.RedvoxPacketM.TimingInformation.SynchExchange.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\" \n\x11TimingScoreMethod\x12\x0b\n\x07UNKNOWN\x10\x00\x1a\xa8\x30\n\x07Sensors\x12>\n\raccelerometer\x18\x01 \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12G\n\x13\x61mbient_temperature\x18\x02 \x01(\x0b\x32*.redvox_api_m.RedvoxPacketM.Sensors.Single\x12\x38\n\x05\x61udio\x18\x03 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.Sensors.Audio\x12M\n\x10\x63ompressed_audio\x18\x04 \x01(\x0b\x32\x33.redvox_api_m.RedvoxPacketM.Sensors.CompressedAudio\x12\x38\n\x07gravity\x18\x05 \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12:\n\tgyroscope\x18\x06 \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12\x38\n\x05image\x18\x07 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.Sensors.Image\x12\x39\n\x05light\x18\x08 \x01(\x0b\x32*.redvox_api_m.RedvoxPacketM.Sensors.Single\x12\x44\n\x13linear_acceleration\x18\t \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12>\n\x08location\x18\n \x01(\x0b\x32,.redvox_api_m.RedvoxPacketM.Sensors.Location\x12=\n\x0cmagnetometer\x18\x0b \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12<\n\x0borientation\x18\x0c \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12<\n\x08pressure\x18\r \x01(\x0b\x32*.redvox_api_m.RedvoxPacketM.Sensors.Single\x12=\n\tproximity\x18\x0e \x01(\x0b\x32*.redvox_api_m.RedvoxPacketM.Sensors.Single\x12\x45\n\x11relative_humidity\x18\x0f \x01(\x0b\x32*.redvox_api_m.RedvoxPacketM.Sensors.Single\x12@\n\x0frotation_vector\x18\x10 \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12\x39\n\x08velocity\x18\x11 \x01(\x0b\x32\'.redvox_api_m.RedvoxPacketM.Sensors.Xyz\x12\x43\n\x08metadata\x18\x12 \x03(\x0b\x32\x31.redvox_api_m.RedvoxPacketM.Sensors.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd3\x02\n\x05\x41udio\x12\x1a\n\x12sensor_description\x18\x01 \x01(\t\x12\x1e\n\x16\x66irst_sample_timestamp\x18\x02 \x01(\x01\x12\x13\n\x0bsample_rate\x18\x03 \x01(\x02\x12\x19\n\x11\x62its_of_precision\x18\x04 \x01(\x02\x12\x14\n\x0cis_scrambled\x18\x05 \x01(\x08\x12\x10\n\x08\x65ncoding\x18\x06 \x01(\t\x12:\n\x07samples\x18\x07 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12I\n\x08metadata\x18\x08 \x03(\x0b\x32\x37.redvox_api_m.RedvoxPacketM.Sensors.Audio.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x8d\x03\n\x0f\x43ompressedAudio\x12\x1a\n\x12sensor_description\x18\x01 \x01(\t\x12\x1e\n\x16\x66irst_sample_timestamp\x18\x02 \x01(\x01\x12\x13\n\x0bsample_rate\x18\x03 \x01(\x02\x12\x14\n\x0cis_scrambled\x18\x04 \x01(\x08\x12\x13\n\x0b\x61udio_bytes\x18\x05 \x01(\x0c\x12S\n\x0b\x61udio_codec\x18\x06 \x01(\x0e\x32>.redvox_api_m.RedvoxPacketM.Sensors.CompressedAudio.AudioCodec\x12S\n\x08metadata\x18\x07 \x03(\x0b\x32\x41.redvox_api_m.RedvoxPacketM.Sensors.CompressedAudio.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"#\n\nAudioCodec\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04\x46LAC\x10\x01\x1a\x9c\x02\n\x06Single\x12\x1a\n\x12sensor_description\x18\x01 \x01(\t\x12=\n\ntimestamps\x18\x02 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12:\n\x07samples\x18\x03 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12J\n\x08metadata\x18\x04 \x03(\x0b\x32\x38.redvox_api_m.RedvoxPacketM.Sensors.Single.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd6\x18\n\x08Location\x12\x1a\n\x12sensor_description\x18\x01 \x01(\t\x12=\n\ntimestamps\x18\x02 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12\x41\n\x0etimestamps_gps\x18\x03 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12I\n\x10latitude_samples\x18\x04 \x01(\x0b\x32/.redvox_api_m.RedvoxPacketM.DoubleSamplePayload\x12J\n\x11longitude_samples\x18\x05 \x01(\x0b\x32/.redvox_api_m.RedvoxPacketM.DoubleSamplePayload\x12\x43\n\x10\x61ltitude_samples\x18\x06 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12@\n\rspeed_samples\x18\x07 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12\x42\n\x0f\x62\x65\x61ring_samples\x18\x08 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12N\n\x1bhorizontal_accuracy_samples\x18\t \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12L\n\x19vertical_accuracy_samples\x18\n \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12I\n\x16speed_accuracy_samples\x18\x0b \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12K\n\x18\x62\x65\x61ring_accuracy_samples\x18\x0c \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12U\n\x12last_best_location\x18\r \x01(\x0b\x32\x39.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation\x12X\n\x15overall_best_location\x18\x0e \x01(\x0b\x32\x39.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation\x12$\n\x1clocation_permissions_granted\x18\x0f \x01(\x08\x12#\n\x1blocation_services_requested\x18\x10 \x01(\x08\x12!\n\x19location_services_enabled\x18\x11 \x01(\x08\x12Y\n\x12location_providers\x18\x12 \x03(\x0e\x32=.redvox_api_m.RedvoxPacketM.Sensors.Location.LocationProvider\x12L\n\x08metadata\x18\x13 \x03(\x0b\x32:.redvox_api_m.RedvoxPacketM.Sensors.Location.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xcf\r\n\x0c\x42\x65stLocation\x12m\n\x1clatitude_longitude_timestamp\x18\x01 \x01(\x0b\x32G.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.BestTimestamp\x12\x63\n\x12\x61ltitude_timestamp\x18\x02 \x01(\x0b\x32G.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.BestTimestamp\x12`\n\x0fspeed_timestamp\x18\x03 \x01(\x0b\x32G.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.BestTimestamp\x12\x62\n\x11\x62\x65\x61ring_timestamp\x18\x04 \x01(\x0b\x32G.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.BestTimestamp\x12\x41\n\x17latitude_longitude_unit\x18\x05 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x37\n\raltitude_unit\x18\x06 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x34\n\nspeed_unit\x18\x07 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x36\n\x0c\x62\x65\x61ring_unit\x18\x08 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12@\n\x16vertical_accuracy_unit\x18\t \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x42\n\x18horizontal_accuracy_unit\x18\n \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12=\n\x13speed_accuracy_unit\x18\x0b \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12?\n\x15\x62\x65\x61ring_accuracy_unit\x18\x0c \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x10\n\x08latitude\x18\r \x01(\x01\x12\x11\n\tlongitude\x18\x0e \x01(\x01\x12\x10\n\x08\x61ltitude\x18\x0f \x01(\x02\x12\r\n\x05speed\x18\x10 \x01(\x02\x12\x0f\n\x07\x62\x65\x61ring\x18\x11 \x01(\x02\x12\x19\n\x11vertical_accuracy\x18\x12 \x01(\x02\x12\x1b\n\x13horizontal_accuracy\x18\x13 \x01(\x02\x12\x16\n\x0espeed_accuracy\x18\x14 \x01(\x02\x12\x18\n\x10\x62\x65\x61ring_accuracy\x18\x15 \x01(\x02\x12\r\n\x05score\x18\x16 \x01(\x02\x12]\n\x06method\x18\x17 \x01(\x0e\x32M.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.LocationScoreMethod\x12X\n\x11location_provider\x18\x18 \x01(\x0e\x32=.redvox_api_m.RedvoxPacketM.Sensors.Location.LocationProvider\x12Y\n\x08metadata\x18\x19 \x03(\x0b\x32G.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xf4\x01\n\rBestTimestamp\x12.\n\x04unit\x18\x01 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x0c\n\x04mach\x18\x02 \x01(\x01\x12\x0b\n\x03gps\x18\x03 \x01(\x01\x12g\n\x08metadata\x18\x04 \x03(\x0b\x32U.redvox_api_m.RedvoxPacketM.Sensors.Location.BestLocation.BestTimestamp.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\")\n\x13LocationScoreMethod\x12\x12\n\x0eUNKNOWN_METHOD\x10\x00\"I\n\x10LocationProvider\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\x08\n\x04USER\x10\x02\x12\x07\n\x03GPS\x10\x03\x12\x0b\n\x07NETWORK\x10\x04\x1a\x94\x03\n\x03Xyz\x12\x1a\n\x12sensor_description\x18\x01 \x01(\t\x12=\n\ntimestamps\x18\x02 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12<\n\tx_samples\x18\x03 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12<\n\ty_samples\x18\x04 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12<\n\tz_samples\x18\x05 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.SamplePayload\x12G\n\x08metadata\x18\x06 \x03(\x0b\x32\x35.redvox_api_m.RedvoxPacketM.Sensors.Xyz.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xf0\x02\n\x05Image\x12\x1a\n\x12sensor_description\x18\x01 \x01(\t\x12=\n\ntimestamps\x18\x02 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12\x0f\n\x07samples\x18\x03 \x03(\x0c\x12I\n\x0bimage_codec\x18\x04 \x01(\x0e\x32\x34.redvox_api_m.RedvoxPacketM.Sensors.Image.ImageCodec\x12I\n\x08metadata\x18\x05 \x03(\x0b\x32\x37.redvox_api_m.RedvoxPacketM.Sensors.Image.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"4\n\nImageCodec\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03PNG\x10\x01\x12\x07\n\x03JPG\x10\x02\x12\x07\n\x03\x42MP\x10\x03\x1a\xf2\x07\n\x0b\x45ventStream\x12\x0c\n\x04name\x18\x01 \x01(\t\x12=\n\ntimestamps\x18\x02 \x01(\x0b\x32).redvox_api_m.RedvoxPacketM.TimingPayload\x12=\n\x06\x65vents\x18\x03 \x03(\x0b\x32-.redvox_api_m.RedvoxPacketM.EventStream.Event\x12G\n\x08metadata\x18\x04 \x03(\x0b\x32\x35.redvox_api_m.RedvoxPacketM.EventStream.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xdc\x05\n\x05\x45vent\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\x12X\n\x0estring_payload\x18\x02 \x03(\x0b\x32@.redvox_api_m.RedvoxPacketM.EventStream.Event.StringPayloadEntry\x12Z\n\x0fnumeric_payload\x18\x03 \x03(\x0b\x32\x41.redvox_api_m.RedvoxPacketM.EventStream.Event.NumericPayloadEntry\x12Z\n\x0f\x62oolean_payload\x18\x04 \x03(\x0b\x32\x41.redvox_api_m.RedvoxPacketM.EventStream.Event.BooleanPayloadEntry\x12T\n\x0c\x62yte_payload\x18\x05 \x03(\x0b\x32>.redvox_api_m.RedvoxPacketM.EventStream.Event.BytePayloadEntry\x12M\n\x08metadata\x18\x06 \x03(\x0b\x32;.redvox_api_m.RedvoxPacketM.EventStream.Event.MetadataEntry\x1a\x34\n\x12StringPayloadEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x13NumericPayloadEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\x1a\x35\n\x13\x42ooleanPayloadEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a\x32\n\x10\x42ytePayloadEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x94\x02\n\rSamplePayload\x12.\n\x04unit\x18\x01 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x0e\n\x06values\x18\x02 \x03(\x02\x12G\n\x10value_statistics\x18\x03 \x01(\x0b\x32-.redvox_api_m.RedvoxPacketM.SummaryStatistics\x12I\n\x08metadata\x18\x04 \x03(\x0b\x32\x37.redvox_api_m.RedvoxPacketM.SamplePayload.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xa0\x02\n\x13\x44oubleSamplePayload\x12.\n\x04unit\x18\x01 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x0e\n\x06values\x18\x02 \x03(\x01\x12G\n\x10value_statistics\x18\x03 \x01(\x0b\x32-.redvox_api_m.RedvoxPacketM.SummaryStatistics\x12O\n\x08metadata\x18\x04 \x03(\x0b\x32=.redvox_api_m.RedvoxPacketM.DoubleSamplePayload.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd1\x02\n\rTimingPayload\x12.\n\x04unit\x18\x01 \x01(\x0e\x32 .redvox_api_m.RedvoxPacketM.Unit\x12\x12\n\ntimestamps\x18\x02 \x03(\x01\x12K\n\x14timestamp_statistics\x18\x03 \x01(\x0b\x32-.redvox_api_m.RedvoxPacketM.SummaryStatistics\x12\x18\n\x10mean_sample_rate\x18\x04 \x01(\x02\x12\x19\n\x11stdev_sample_rate\x18\x05 \x01(\x02\x12I\n\x08metadata\x18\x06 \x03(\x0b\x32\x37.redvox_api_m.RedvoxPacketM.TimingPayload.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xf5\x01\n\x11SummaryStatistics\x12\r\n\x05\x63ount\x18\x01 \x01(\x01\x12\x0c\n\x04mean\x18\x02 \x01(\x01\x12\x1a\n\x12standard_deviation\x18\x03 \x01(\x01\x12\x0b\n\x03min\x18\x04 \x01(\x01\x12\x0b\n\x03max\x18\x05 \x01(\x01\x12\r\n\x05range\x18\x06 \x01(\x01\x12M\n\x08metadata\x18\x07 \x03(\x0b\x32;.redvox_api_m.RedvoxPacketM.SummaryStatistics.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x83\x03\n\x04Unit\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x1d\n\x19METERS_PER_SECOND_SQUARED\x10\x01\x12\x0e\n\nKILOPASCAL\x10\x02\x12\x16\n\x12RADIANS_PER_SECOND\x10\x03\x12\x13\n\x0f\x44\x45\x43IMAL_DEGREES\x10\x04\x12\n\n\x06METERS\x10\x05\x12\x15\n\x11METERS_PER_SECOND\x10\x06\x12\x0e\n\nMICROTESLA\x10\x07\x12\x19\n\x15LSB_PLUS_MINUS_COUNTS\x10\x08\x12!\n\x1dMICROSECONDS_SINCE_UNIX_EPOCH\x10\t\x12\x0b\n\x07\x44\x45\x43IBEL\x10\n\x12\x13\n\x0f\x44\x45GREES_CELSIUS\x10\x0b\x12\x08\n\x04\x42YTE\x10\x0c\x12\x0e\n\nPERCENTAGE\x10\r\x12\x0b\n\x07RADIANS\x10\x0e\x12\x10\n\x0cMICROAMPERES\x10\x0f\x12\x0f\n\x0b\x43\x45NTIMETERS\x10\x10\x12\x15\n\x11NORMALIZED_COUNTS\x10\x11\x12\x07\n\x03LUX\x10\x12\x12\x0c\n\x08UNITLESS\x10\x13\x12\x07\n\x03PCM\x10\x14\"\xac\x01\n\x16\x45ncryptedRedvoxPacketM\x12\x0e\n\x06header\x18\x01 \x01(\x0c\x12\x0e\n\x06packet\x18\x02 \x01(\x0c\x1ar\n\x06Header\x12\x12\n\nstation_id\x18\x01 \x01(\t\x12\x14\n\x0cstation_uuid\x18\x02 \x01(\t\x12\x12\n\nauth_token\x18\x03 \x01(\t\x12\x16\n\x0e\x66irebase_token\x18\x04 \x01(\t\x12\x12\n\nauth_email\x18\x05 \x01(\t\"\x89\x01\n\x12\x41\x63quisitionRequest\x12\x12\n\nauth_token\x18\x01 \x01(\t\x12\x16\n\x0e\x66irebase_token\x18\x02 \x01(\t\x12\x10\n\x08\x63hecksum\x18\x03 \x01(\x03\x12\x14\n\x0cis_encrypted\x18\x04 \x01(\x08\x12\x0f\n\x07payload\x18\x05 \x01(\x0c\x12\x0e\n\x06seq_id\x18\x06 \x01(\x03\"\xf5\x01\n\x13\x41\x63quisitionResponse\x12\x45\n\rresponse_type\x18\x01 \x01(\x0e\x32..redvox_api_m.AcquisitionResponse.ResponseType\x12\x10\n\x08\x63hecksum\x18\x02 \x01(\x03\x12\x0f\n\x07\x64\x65tails\x18\x03 \x01(\t\x12\x0e\n\x06resend\x18\x04 \x01(\x08\x12\x0e\n\x06seq_id\x18\x05 \x01(\x03\"T\n\x0cResponseType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x06\n\x02OK\x10\x01\x12\x0e\n\nAUTH_ERROR\x10\x02\x12\x0e\n\nDATA_ERROR\x10\x03\x12\x0f\n\x0bOTHER_ERROR\x10\x04\"\\\n\x0cSynchRequest\x12\x12\n\nstation_id\x18\x01 \x01(\t\x12\x14\n\x0cstation_uuid\x18\x02 \x01(\t\x12\x0e\n\x06seq_id\x18\x03 \x01(\r\x12\x12\n\nsub_seq_id\x18\x04 \x01(\r\"\x85\x01\n\rSynchResponse\x12\x12\n\nstation_id\x18\x01 \x01(\t\x12\x14\n\x0cstation_uuid\x18\x02 \x01(\t\x12\x0e\n\x06seq_id\x18\x03 \x01(\r\x12\x12\n\nsub_seq_id\x18\x04 \x01(\r\x12\x12\n\nrecv_ts_us\x18\x05 \x01(\x04\x12\x12\n\nsend_ts_us\x18\x06 \x01(\x04\x42\x10\n\x0eio.redvox.apisb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'src.redvox_api_m.redvox_api_m_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\016io.redvox.apis'
  _REDVOXPACKETM_METADATAENTRY._options = None
  _REDVOXPACKETM_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_STATIONINFORMATION_METADATAENTRY._options = None
  _REDVOXPACKETM_STATIONINFORMATION_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_STATIONINFORMATION_SERVICEURLS_METADATAENTRY._options = None
  _REDVOXPACKETM_STATIONINFORMATION_SERVICEURLS_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_METADATAENTRY._options = None
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_METADATAENTRY._options = None
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_TIMINGINFORMATION_METADATAENTRY._options = None
  _REDVOXPACKETM_TIMINGINFORMATION_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_TIMINGINFORMATION_SYNCHEXCHANGE_METADATAENTRY._options = None
  _REDVOXPACKETM_TIMINGINFORMATION_SYNCHEXCHANGE_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_AUDIO_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_AUDIO_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_SINGLE_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_SINGLE_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_LOCATION_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_LOCATION_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_BESTTIMESTAMP_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_BESTTIMESTAMP_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_XYZ_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_XYZ_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SENSORS_IMAGE_METADATAENTRY._options = None
  _REDVOXPACKETM_SENSORS_IMAGE_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_EVENTSTREAM_METADATAENTRY._options = None
  _REDVOXPACKETM_EVENTSTREAM_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_EVENTSTREAM_EVENT_STRINGPAYLOADENTRY._options = None
  _REDVOXPACKETM_EVENTSTREAM_EVENT_STRINGPAYLOADENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_EVENTSTREAM_EVENT_NUMERICPAYLOADENTRY._options = None
  _REDVOXPACKETM_EVENTSTREAM_EVENT_NUMERICPAYLOADENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BOOLEANPAYLOADENTRY._options = None
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BOOLEANPAYLOADENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BYTEPAYLOADENTRY._options = None
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BYTEPAYLOADENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_EVENTSTREAM_EVENT_METADATAENTRY._options = None
  _REDVOXPACKETM_EVENTSTREAM_EVENT_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SAMPLEPAYLOAD_METADATAENTRY._options = None
  _REDVOXPACKETM_SAMPLEPAYLOAD_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_DOUBLESAMPLEPAYLOAD_METADATAENTRY._options = None
  _REDVOXPACKETM_DOUBLESAMPLEPAYLOAD_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_TIMINGPAYLOAD_METADATAENTRY._options = None
  _REDVOXPACKETM_TIMINGPAYLOAD_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM_SUMMARYSTATISTICS_METADATAENTRY._options = None
  _REDVOXPACKETM_SUMMARYSTATISTICS_METADATAENTRY._serialized_options = b'8\001'
  _REDVOXPACKETM._serialized_start=54
  _REDVOXPACKETM._serialized_end=14882
  _REDVOXPACKETM_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_STATIONINFORMATION._serialized_start=482
  _REDVOXPACKETM_STATIONINFORMATION._serialized_end=5171
  _REDVOXPACKETM_STATIONINFORMATION_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_STATIONINFORMATION_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_STATIONINFORMATION_SERVICEURLS._serialized_start=1107
  _REDVOXPACKETM_STATIONINFORMATION_SERVICEURLS._serialized_end=1332
  _REDVOXPACKETM_STATIONINFORMATION_SERVICEURLS_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_STATIONINFORMATION_SERVICEURLS_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS._serialized_start=1335
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS._serialized_end=2992
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_NETWORKTYPE._serialized_start=2589
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_NETWORKTYPE._serialized_end=2674
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_WIFIWAKELOCK._serialized_start=2676
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_WIFIWAKELOCK._serialized_end=2743
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_CELLSERVICESTATE._serialized_start=2745
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_CELLSERVICESTATE._serialized_end=2839
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_POWERSTATE._serialized_start=2841
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_POWERSTATE._serialized_end=2920
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_SCREENSTATE._serialized_start=2922
  _REDVOXPACKETM_STATIONINFORMATION_STATIONMETRICS_SCREENSTATE._serialized_end=2992
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS._serialized_start=2995
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS._serialized_end=5020
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_FFTOVERLAP._serialized_start=4307
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_FFTOVERLAP._serialized_end=4380
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_AUDIOSAMPLINGRATE._serialized_start=4382
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_AUDIOSAMPLINGRATE._serialized_end=4492
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_AUDIOSOURCETUNING._serialized_start=4494
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_AUDIOSOURCETUNING._serialized_end=4596
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_INPUTSENSOR._serialized_start=4599
  _REDVOXPACKETM_STATIONINFORMATION_APPSETTINGS_INPUTSENSOR._serialized_end=5020
  _REDVOXPACKETM_STATIONINFORMATION_OSTYPE._serialized_start=5022
  _REDVOXPACKETM_STATIONINFORMATION_OSTYPE._serialized_end=5101
  _REDVOXPACKETM_STATIONINFORMATION_METRICSRATE._serialized_start=5103
  _REDVOXPACKETM_STATIONINFORMATION_METRICSRATE._serialized_end=5171
  _REDVOXPACKETM_TIMINGINFORMATION._serialized_start=5174
  _REDVOXPACKETM_TIMINGINFORMATION._serialized_end=6134
  _REDVOXPACKETM_TIMINGINFORMATION_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_TIMINGINFORMATION_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_TIMINGINFORMATION_SYNCHEXCHANGE._serialized_start=5823
  _REDVOXPACKETM_TIMINGINFORMATION_SYNCHEXCHANGE._serialized_end=6100
  _REDVOXPACKETM_TIMINGINFORMATION_SYNCHEXCHANGE_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_TIMINGINFORMATION_SYNCHEXCHANGE_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_TIMINGINFORMATION_TIMINGSCOREMETHOD._serialized_start=6102
  _REDVOXPACKETM_TIMINGINFORMATION_TIMINGSCOREMETHOD._serialized_end=6134
  _REDVOXPACKETM_SENSORS._serialized_start=6137
  _REDVOXPACKETM_SENSORS._serialized_end=12321
  _REDVOXPACKETM_SENSORS_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_AUDIO._serialized_start=7356
  _REDVOXPACKETM_SENSORS_AUDIO._serialized_end=7695
  _REDVOXPACKETM_SENSORS_AUDIO_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_AUDIO_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO._serialized_start=7698
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO._serialized_end=8095
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO_AUDIOCODEC._serialized_start=8060
  _REDVOXPACKETM_SENSORS_COMPRESSEDAUDIO_AUDIOCODEC._serialized_end=8095
  _REDVOXPACKETM_SENSORS_SINGLE._serialized_start=8098
  _REDVOXPACKETM_SENSORS_SINGLE._serialized_end=8382
  _REDVOXPACKETM_SENSORS_SINGLE_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_SINGLE_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_LOCATION._serialized_start=8385
  _REDVOXPACKETM_SENSORS_LOCATION._serialized_end=11543
  _REDVOXPACKETM_SENSORS_LOCATION_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_LOCATION_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION._serialized_start=9725
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION._serialized_end=11468
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_BESTTIMESTAMP._serialized_start=11181
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_BESTTIMESTAMP._serialized_end=11425
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_BESTTIMESTAMP_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_BESTTIMESTAMP_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_LOCATIONSCOREMETHOD._serialized_start=11427
  _REDVOXPACKETM_SENSORS_LOCATION_BESTLOCATION_LOCATIONSCOREMETHOD._serialized_end=11468
  _REDVOXPACKETM_SENSORS_LOCATION_LOCATIONPROVIDER._serialized_start=11470
  _REDVOXPACKETM_SENSORS_LOCATION_LOCATIONPROVIDER._serialized_end=11543
  _REDVOXPACKETM_SENSORS_XYZ._serialized_start=11546
  _REDVOXPACKETM_SENSORS_XYZ._serialized_end=11950
  _REDVOXPACKETM_SENSORS_XYZ_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_XYZ_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_IMAGE._serialized_start=11953
  _REDVOXPACKETM_SENSORS_IMAGE._serialized_end=12321
  _REDVOXPACKETM_SENSORS_IMAGE_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SENSORS_IMAGE_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SENSORS_IMAGE_IMAGECODEC._serialized_start=12269
  _REDVOXPACKETM_SENSORS_IMAGE_IMAGECODEC._serialized_end=12321
  _REDVOXPACKETM_EVENTSTREAM._serialized_start=12324
  _REDVOXPACKETM_EVENTSTREAM._serialized_end=13334
  _REDVOXPACKETM_EVENTSTREAM_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_EVENTSTREAM_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_EVENTSTREAM_EVENT._serialized_start=12602
  _REDVOXPACKETM_EVENTSTREAM_EVENT._serialized_end=13334
  _REDVOXPACKETM_EVENTSTREAM_EVENT_STRINGPAYLOADENTRY._serialized_start=13071
  _REDVOXPACKETM_EVENTSTREAM_EVENT_STRINGPAYLOADENTRY._serialized_end=13123
  _REDVOXPACKETM_EVENTSTREAM_EVENT_NUMERICPAYLOADENTRY._serialized_start=13125
  _REDVOXPACKETM_EVENTSTREAM_EVENT_NUMERICPAYLOADENTRY._serialized_end=13178
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BOOLEANPAYLOADENTRY._serialized_start=13180
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BOOLEANPAYLOADENTRY._serialized_end=13233
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BYTEPAYLOADENTRY._serialized_start=13235
  _REDVOXPACKETM_EVENTSTREAM_EVENT_BYTEPAYLOADENTRY._serialized_end=13285
  _REDVOXPACKETM_EVENTSTREAM_EVENT_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_EVENTSTREAM_EVENT_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SAMPLEPAYLOAD._serialized_start=13337
  _REDVOXPACKETM_SAMPLEPAYLOAD._serialized_end=13613
  _REDVOXPACKETM_SAMPLEPAYLOAD_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SAMPLEPAYLOAD_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_DOUBLESAMPLEPAYLOAD._serialized_start=13616
  _REDVOXPACKETM_DOUBLESAMPLEPAYLOAD._serialized_end=13904
  _REDVOXPACKETM_DOUBLESAMPLEPAYLOAD_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_DOUBLESAMPLEPAYLOAD_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_TIMINGPAYLOAD._serialized_start=13907
  _REDVOXPACKETM_TIMINGPAYLOAD._serialized_end=14244
  _REDVOXPACKETM_TIMINGPAYLOAD_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_TIMINGPAYLOAD_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_SUMMARYSTATISTICS._serialized_start=14247
  _REDVOXPACKETM_SUMMARYSTATISTICS._serialized_end=14492
  _REDVOXPACKETM_SUMMARYSTATISTICS_METADATAENTRY._serialized_start=432
  _REDVOXPACKETM_SUMMARYSTATISTICS_METADATAENTRY._serialized_end=479
  _REDVOXPACKETM_UNIT._serialized_start=14495
  _REDVOXPACKETM_UNIT._serialized_end=14882
  _ENCRYPTEDREDVOXPACKETM._serialized_start=14885
  _ENCRYPTEDREDVOXPACKETM._serialized_end=15057
  _ENCRYPTEDREDVOXPACKETM_HEADER._serialized_start=14943
  _ENCRYPTEDREDVOXPACKETM_HEADER._serialized_end=15057
  _ACQUISITIONREQUEST._serialized_start=15060
  _ACQUISITIONREQUEST._serialized_end=15197
  _ACQUISITIONRESPONSE._serialized_start=15200
  _ACQUISITIONRESPONSE._serialized_end=15445
  _ACQUISITIONRESPONSE_RESPONSETYPE._serialized_start=15361
  _ACQUISITIONRESPONSE_RESPONSETYPE._serialized_end=15445
  _SYNCHREQUEST._serialized_start=15447
  _SYNCHREQUEST._serialized_end=15539
  _SYNCHRESPONSE._serialized_start=15542
  _SYNCHRESPONSE._serialized_end=15675
# @@protoc_insertion_point(module_scope)



"""
This module provides methods and datatypes for the efficient extraction and manipulation of machine learning data stored
 in RedVox packet EventStreams.
"""

from dataclasses import dataclass
from enum import Enum
from math import isfinite
from typing import Optional, Dict, List

import numpy as np
from redvox.common.errors import RedVoxError

from redvox.api1000.common.generic import ProtoRepeatedMessage
from redvox.api1000.wrapped_redvox_packet.event_streams import EventStream, Event
from redvox.api1000.wrapped_redvox_packet.wrapped_packet import WrappedRedvoxPacketM

ML_CLASS_PREFIX: str = "class_"
ML_SCORE_PREFIX: str = "score_"
ML_EVENT_STREAM_NAME: str = "inference"
ML_METADATA_MODEL_VERSION_KEY: str = "model_version"
ML_METADATA_INPUT_SAMPLES_PER_HOP_KEY: str = "input_samples_per_hop"
ML_METADATA_INPUT_SAMPLE_RATE_KEY: str = "input_sample_rate"
ML_METADATA_INPUT_SAMPLES_PER_WINDOW_KEY: str = "input_samples_per_window"


class MlError(RedVoxError):
    def __init__(self, msg: str):
        super().__init__(f"MlError: {msg}")


@dataclass
class ExtractedMl:
    """
    Contains the extracted ML classes and scores as well as metadata relating to the utilized model.
    """

    metadata: "MlMetadata"
    windows: List["MlWindow"]

    def sort(self, sort_by: "SortBy") -> "ExtractedMl":
        """
        Sorts the labels in ascending or descending order by either score of class name.
        :param sort_by: The sort operation to use.
        :return: An updated instance of ExtractedMl.
        """
        self.windows = list(map(lambda window: window.sort(sort_by), self.windows))
        return self

    def prune_zeros(self) -> "ExtractedMl":
        """
        Removes labels that have a score of 0.
        :return: An updated instance of ExtractedMl.
        """
        self.windows = list(map(lambda window: window.prune_zeros(), self.windows))
        return self

    def prune_lt(self, min_v: float) -> "ExtractedMl":
        """
        Prunes labels with score less than the provided minimum.
        :param min_v: The minimum acceptable label score.
        :return: An updated instance of ExtractedMl.
        """
        if min_v <= 0:
            raise MlError(f"min_v={min_v} must be > 0")

        self.windows = list(map(lambda window: window.prune_lt(min_v), self.windows))
        return self

    def retain_top(self, n: int) -> "ExtractedMl":
        """
        Sorts labels in descending order by score and only keeps up to the top n labels.
        :param n: The number of labels to keep.
        :return: An updated instance of ExtractedMl.
        """
        if n <= 0:
            raise MlError(f"n={n} must be > 0")

        self.windows = list(map(lambda window: window.retain_top(n), self.windows))
        return self


@dataclass
class MlMetadata:
    """
    Metadata relating to the utilized ML model.
    """

    model_name: str
    model_version: str
    input_samples_per_hop: int
    input_sample_rate: int
    input_samples_per_window: int


@dataclass
class MlWindow:
    """
    Labels from a single time window.
    """

    timestamp: int
    labels: List["Label"]

    def sort(self, sort_by: "SortBy") -> "MlWindow":
        """
        Sorts the labels in ascending or descending order by either score of class name.
        :param sort_by: The sort operation to use.
        :return: An updated instance of MlWindow.
        """
        if sort_by is SortBy.CLASS_ASC:
            self.labels = sorted(self.labels, key=lambda label: label.class_name)
        elif sort_by is SortBy.CLASS_DESC:
            self.labels = sorted(
                self.labels, key=lambda label: label.class_name, reverse=True
            )
        elif sort_by is SortBy.SCORE_ASC:
            self.labels = sorted(self.labels, key=lambda label: label.score)
        else:
            self.labels = sorted(
                self.labels, key=lambda label: label.score, reverse=True
            )

        return self

    def prune_zeros(self) -> "MlWindow":
        """
        Removes labels that have a score of 0.
        :return: An updated instance of MlWindow.
        """
        self.labels = list(filter(lambda label: label.score > 0, self.labels))
        return self

    def prune_lt(self, min_v: float) -> "MlWindow":
        """
        Prunes labels with score less than the provided minimum.
        :param min_v: The minimum acceptable label score.
        :return: An updated instance of MlWindow.
        """
        if min_v <= 0:
            raise MlError(f"min_v={min_v} must be > 0")

        self.labels = list(filter(lambda label: label.score >= min_v, self.labels))
        return self

    def retain_top(self, n: int) -> "MlWindow":
        """
        Sorts labels in descending order by score and only keeps up to the top n labels.
        :param n: The number of labels to keep.
        :return: An updated instance of MlWindow.
        """
        if n <= 0:
            raise MlError(f"n={n} must be > 0")

        sorted_window: MlWindow = self.sort(SortBy.SCORE_DESC)
        sorted_window.labels = sorted_window.labels[:n]
        return sorted_window


@dataclass
class Label:
    """
    A pair containing a class name and a score.
    """

    class_name: str
    score: float


class SortBy(Enum):
    """
    An enumeration that represents the valid label sorting methods.
    """

    SCORE_ASC: int = 1
    SCORE_DESC: int = 2
    CLASS_ASC: int = 3
    CLASS_DESC: int = 4


def extract_ml_metadata(stream: EventStream) -> MlMetadata:
    """
    Extracts ML metadata from an event stream.
    :param stream: The event stream to extract the ML metadata from.
    :return: An instance of MlMetadata.
    """
    if stream.get_name() != ML_EVENT_STREAM_NAME:
        raise MlError(
            f"Invalid ML event stream name={stream.get_name()} != {ML_EVENT_STREAM_NAME}"
        )

    if stream.get_events().get_count() == 0:
        raise MlError("ML EventStream contains 0 Events")

    name: str = stream.get_events().get_values()[0].get_description()
    meta: Dict[str, str] = stream.get_metadata().get_metadata()

    for key in [
        ML_METADATA_MODEL_VERSION_KEY,
        ML_METADATA_INPUT_SAMPLES_PER_HOP_KEY,
        ML_METADATA_INPUT_SAMPLE_RATE_KEY,
        ML_METADATA_INPUT_SAMPLES_PER_WINDOW_KEY,
    ]:
        if key not in meta:
            raise MlError(f"Missing required ML metadata key={key}")

    try:
        return MlMetadata(
            name,
            meta[ML_METADATA_MODEL_VERSION_KEY],
            int(meta[ML_METADATA_INPUT_SAMPLES_PER_HOP_KEY]),
            int(meta[ML_METADATA_INPUT_SAMPLE_RATE_KEY]),
            int(meta[ML_METADATA_INPUT_SAMPLES_PER_WINDOW_KEY]),
        )
    except ValueError:
        raise MlError("Could not parse ML metadata")


def find_ml_event_stream(packet: WrappedRedvoxPacketM) -> Optional[EventStream]:
    """
    Attempts to find an event stream with ML data.
    :param packet: The packet to search in.
    :return: An instance of the matching event stream or None.
    """
    streams: ProtoRepeatedMessage = packet.get_event_streams()

    stream: EventStream
    for stream in streams.get_values():
        if stream.get_name() == ML_EVENT_STREAM_NAME:
            return stream

    return None


def label_at(
    str_payload: Dict[str, str], num_payload: Dict[str, float], idx: int
) -> Label:
    """
    Finds the label and score in the event payloads.
    :param str_payload: The event string payload.
    :param num_payload: The event numeric payload.
    :param idx: The index of the class and score.
    :return: An instance of a Label.
    """
    class_key: str = f"{ML_CLASS_PREFIX}{idx}"
    score_key: str = f"{ML_SCORE_PREFIX}{idx}"

    if class_key not in str_payload:
        raise MlError(f"Missing required class_key={class_key}")

    if score_key not in num_payload:
        raise MlError(f"Missing required score_key={score_key}")

    class_name: str = str_payload[class_key]
    score: float = num_payload[score_key]

    if not isfinite(score):
        raise MlError(f"Invalid non-finite score={score}")

    return Label(class_name, score)


def extract_ml_windows(stream: EventStream) -> List[MlWindow]:
    """
    Extracts ML windows from an event stream.
    :param stream: The stream to extract windows from.
    :return: A list of ML windows.
    """
    if stream.get_name() != ML_EVENT_STREAM_NAME:
        raise MlError(
            f"Invalid ML event stream name={stream.get_name()} != {ML_EVENT_STREAM_NAME}"
        )

    timestamps: np.ndarray = stream.get_timestamps().get_timestamps()
    windows: List[MlWindow] = []

    events: List[Event] = stream.get_events().get_values()

    if len(timestamps) != len(events):
        raise MlError(f"len(timestamps={len(timestamps)}) != len(events={len(events)})")

    idx_window: int
    timestamp: float
    for (idx_window, timestamp) in enumerate(timestamps):
        event: Event = events[idx_window]
        str_payload: Dict[str, str] = event.get_string_payload().get_metadata()
        num_payload: Dict[str, float] = event.get_numeric_payload().get_metadata()

        labels: List[Label] = []
        for i in range(len(str_payload)):
            label: Label = label_at(str_payload, num_payload, i)
            labels.append(label)

        window: MlWindow = MlWindow(int(round(timestamp)), labels)
        windows.append(window)

    return windows


def extract_ml_from_event_stream(stream: EventStream) -> ExtractedMl:
    """
    Extract ML parameters from an event stream.
    :param stream: The event stream to extract ML parameters from.
    :return: The extracted ML parameters or None.
    """
    if stream.get_name() != ML_EVENT_STREAM_NAME:
        raise MlError(
            f"Invalid ML event stream name={stream.get_name()} != {ML_EVENT_STREAM_NAME}"
        )

    metadata: MlMetadata = extract_ml_metadata(stream)
    windows: List[MlWindow] = extract_ml_windows(stream)
    return ExtractedMl(metadata, windows)


def extract_ml_from_packet(packet: WrappedRedvoxPacketM) -> Optional[ExtractedMl]:
    """
    Extract ML parameters from an event stream.
    :param packet: The packet to extract ML parameters from.
    :return: The extracted ML parameters or None.
    """
    stream: Optional[EventStream] = find_ml_event_stream(packet)
    if stream is None:
        return None
    return extract_ml_from_event_stream(stream)


def extract_ml_from_file(file_path: str) -> Optional[ExtractedMl]:
    """
    Extract ML parameters from a file.
    :param file_path: The path of the packet to extract ML parameters from.
    :return: The extracted ML parameters or None.
    """
    packet: WrappedRedvoxPacketM = WrappedRedvoxPacketM.from_compressed_path(file_path)
    return extract_ml_from_packet(packet)



import os.path
from pathlib import Path
from typing import Dict, List, Optional

import numpy as np

import redvox
import redvox.api1000.proto.redvox_api_m_pb2 as api_m
import redvox.common.date_time_utils as dtu
import redvox.common.session_io as s_io
import redvox.common.session_model_utils as smu
from redvox.cloud import session_model_api as cloud_sm
from redvox.common.errors import RedVoxError, RedVoxExceptions


SESSION_VERSION = "2023-06-28"  # Version of the SessionModel
CLIENT_NAME = "redvox-sdk/session_model"  # Name of the client used to create the SessionModel
CLIENT_VERSION = SESSION_VERSION  # Version of the client used to create the SessionModel
APP_NAME = "RedVox"  # Default name of the app
DAILY_SESSION_NAME = "Day"  # Identifier for day-long dynamic sessions
HOURLY_SESSION_NAME = "Hour"  # Identifier for hour-long dynamic sessions


class SessionModel:
    """
    SDK version of Session from the cloud API
    """

    def __init__(
        self, session: Optional[cloud_sm.Session] = None, dynamic: Optional[Dict[str, cloud_sm.DynamicSession]] = None
    ):
        self.cloud_session: Optional[cloud_sm.Session] = session
        self.dynamic_sessions: Dict[str, cloud_sm.DynamicSession] = {} if dynamic is None else dynamic
        self._sdk_version: str = redvox.VERSION
        self._errors: RedVoxExceptions = RedVoxExceptions("SessionModel")

    def __repr__(self):
        return (
            f"cloud_session: {self.cloud_session}, "
            f"dynamic_sessions: {self.dynamic_sessions}, "
            f"sdk_version: {self._sdk_version}"
            f"errors: {self._errors}"
        )

    def as_dict(self) -> dict:
        """
        :return: SessionModel as dictionary
        """
        return {
            "cloud_session": self.cloud_session.to_dict(),
            "dynamic_sessions": {n: m.to_dict() for n, m in self.dynamic_sessions.items()},
        }

    @staticmethod
    def from_dict(dictionary: Dict) -> "SessionModel":
        """
        :param dictionary: dictionary to read from
        :return: SessionModel from the dict
        """
        return SessionModel(
            cloud_sm.Session.from_dict(dictionary["cloud_session"]),
            {n: cloud_sm.DynamicSession.from_dict(m) for n, m in dictionary["dynamic_sessions"].items()},
        )

    def compress(self, out_dir: str = ".") -> Path:
        """
        Compresses this SessionModel to a file at out_dir.
        Uses the id and start_ts to name the file.

        :param out_dir: Directory to save file to.  Default "." (current directory)
        :return: The path to the written file.
        """
        return s_io.compress_session_model(self, out_dir)

    def save(self, out_type: str = "json", out_dir: str = ".") -> Path:
        """
        Save the SessionModel to disk.  Options for out_type are "json" for JSON file and "pkl" for .pkl file.
        Defaults to "json".  File will be named after id and start_ts of the SessionModel

        :param out_type: "json" for JSON file and "pkl" for .pkl file
        :param out_dir: Directory to save file to.  Default "." (current directory)
        :return: path to saved file
        """
        if out_type == "pkl":
            return self.compress(out_dir)
        return s_io.session_model_to_json_file(self, out_dir)

    @staticmethod
    def load(file_path: str) -> "SessionModel":
        """
        Load only works on a JSON or .pkl file.

        :param file_path: full name and path to the SessionModel file
        :return: SessionModel from a JSON or .pkl file.
        """
        ext = os.path.splitext(file_path)[1]
        if ext == ".json":
            return SessionModel.from_dict(s_io.session_model_dict_from_json_file(file_path))
        elif ext == ".pkl":
            return s_io.decompress_session_model(file_path)
        else:
            raise ValueError(f"{file_path} has unknown file extension; this function only accepts json and pkl files.")

    def default_file_name(self) -> str:
        """
        :return: Default file name as [id]_[start_ts]_model, with start_ts as integer of microseconds
                    since epoch UTC.  File extension NOT included.
        """
        return (
            f"{self.cloud_session.id}_"
            f"{0 if np.isnan(self.cloud_session.start_ts) else self.cloud_session.start_ts}_model"
        )

    @staticmethod
    def create_from_packet(packet: api_m.RedvoxPacketM) -> "SessionModel":
        """
        :param packet: API M packet of data to read
        :return: Session using the data from the packet
        """
        try:
            duration = (
                packet.timing_information.packet_end_mach_timestamp
                - packet.timing_information.packet_start_mach_timestamp
            )
            all_sensors = smu.get_all_sensors_in_packet(packet)
            sensors = [cloud_sm.Sensor(s[0], s[1], smu.add_to_stats(s[2])) for s in all_sensors]
            local_ts = smu.get_local_timesync(packet)
            if local_ts is None:
                raise RedVoxError(
                    f"Unable to find timing data for station {packet.station_information.id}.\n"
                    f"Timing is required to complete SessionModel.\nNow Quitting."
                )
            fst_lst = cloud_sm.FirstLastBufTimeSync([], smu.NUM_BUFFER_POINTS, [], smu.NUM_BUFFER_POINTS)
            for f in local_ts[5]:
                smu.add_to_fst_buffer(fst_lst.fst, fst_lst.fst_max_size, f.ts, f)
                smu.add_to_lst_buffer(fst_lst.lst, fst_lst.lst_max_size, f.ts, f)
            timing = cloud_sm.Timing(local_ts[0], local_ts[1], local_ts[2], local_ts[3], local_ts[4], fst_lst)
            result = SessionModel(
                cloud_sm.Session(
                    id=packet.station_information.id,
                    uuid=packet.station_information.uuid,
                    desc=packet.station_information.description,
                    start_ts=int(packet.timing_information.app_start_mach_timestamp),
                    client=CLIENT_NAME,
                    client_ver=CLIENT_VERSION,
                    session_ver=SESSION_VERSION,
                    app=APP_NAME,
                    api=int(packet.api),
                    sub_api=int(packet.sub_api),
                    make=packet.station_information.make,
                    model=packet.station_information.model,
                    app_ver=packet.station_information.app_version,
                    owner=packet.station_information.auth_id,
                    private=packet.station_information.is_private,
                    packet_dur=duration,
                    sensors=sensors,
                    n_pkts=1,
                    timing=timing,
                    sub=[],
                )
            )
            result.sub = [result.add_dynamic_day(packet)]
        except Exception as e:
            # result = SessionModel(station_description=f"FAILED: {e}")
            raise e
        return result

    @staticmethod
    def create_from_stream(data_stream: List[api_m.RedvoxPacketM]) -> "SessionModel":
        """
        :param data_stream: list of API M packets from a single station to read
        :return: SessionModel using the data packets from the stream
        """
        p1 = data_stream.pop(0)
        model = SessionModel.create_from_packet(p1)
        for p in data_stream:
            model.add_data_from_packet(p)
        data_stream.insert(0, p1)
        return model

    def add_data_from_packet(self, packet: api_m.RedvoxPacketM):
        """
        Adds the data from the packet to the SessionModel.
        If the packet doesn't match the key of the SessionModel, writes an error and no data is added

        :param packet: packet to add
        """
        if (
            self.cloud_session.session_key() != f"{packet.station_information.id}:{packet.station_information.uuid}:"
            f"{int(packet.timing_information.app_start_mach_timestamp)}"
        ):
            self._errors.append(
                "Attempted to add packet with invalid key: "
                f"{packet.station_information.id}:{packet.station_information.uuid}:"
                f"{int(packet.timing_information.app_start_mach_timestamp)}"
            )
            return
        local_ts = smu.get_local_timesync(packet)
        if local_ts is None:
            self._errors.append(
                f"Timesync doesn't exist in packet starting at "
                f"{packet.timing_information.packet_start_mach_timestamp}."
            )
        else:
            timing = self.cloud_session.timing
            for f in local_ts[5]:
                smu.add_to_fst_buffer(timing.fst_lst.fst, timing.fst_lst.fst_max_size, f.ts, f)
                smu.add_to_lst_buffer(timing.fst_lst.lst, timing.fst_lst.lst_max_size, f.ts, f)
            timing.n_ex += local_ts[2]
            timing.mean_lat = (timing.mean_lat * self.cloud_session.n_pkts + local_ts[3]) / (
                self.cloud_session.n_pkts + 1
            )
            timing.mean_off = (timing.mean_off * self.cloud_session.n_pkts + local_ts[4]) / (
                self.cloud_session.n_pkts + 1
            )
            if local_ts[0] < timing.first_data_ts:
                timing.first_data_ts = local_ts[0]
            if local_ts[1] > timing.last_data_ts:
                timing.last_data_ts = local_ts[1]
        all_sensors = smu.get_all_sensors_in_packet(packet)
        for s in all_sensors:
            sensor = self.get_sensor(s[0], s[1])
            if sensor is not None:
                sensor.sample_rate_stats = smu.add_to_stats(s[2], sensor.sample_rate_stats)
            else:
                self.cloud_session.sensors.append(cloud_sm.Sensor(s[0], s[1], smu.add_to_stats(s[2])))
        self.add_dynamic_day(packet)
        self.cloud_session.n_pkts += 1

    def add_dynamic_hour(self, data: dict, packet_start: float, session_key: str) -> str:
        """
        Add (or update an existing session if key exists) a dynamic session with length of 1 hour using a single packet

        :param data: dictionary of data to add
        :param packet_start: starting timestamp of the packet in microseconds since epoch UTC
        :param session_key: the session key of the parent Session
        :return: the key to the new dynamic session
        """
        start_dt = dtu.datetime_from_epoch_microseconds_utc(packet_start)
        hour_start_dt = dtu.datetime(start_dt.year, start_dt.month, start_dt.day, start_dt.hour)
        hour_end_ts = int(dtu.datetime_to_epoch_microseconds_utc(hour_start_dt + dtu.timedelta(hours=1)))
        hour_start_ts = int(dtu.datetime_to_epoch_microseconds_utc(hour_start_dt))
        key = f"{session_key}:{hour_start_ts}:{hour_end_ts}"
        if key in self.dynamic_sessions.keys():
            self._update_dynamic_session(key, data, [f"{packet_start}"])
        else:
            self.dynamic_sessions[key] = cloud_sm.DynamicSession(
                1,
                smu.add_location_data(data["location"]),
                smu.add_to_stats(data["battery"]),
                smu.add_to_stats(data["temperature"]),
                session_key,
                hour_start_ts,
                hour_end_ts,
                HOURLY_SESSION_NAME,
                [f"{packet_start}"],
            )
        return key

    def add_dynamic_day(self, packet: api_m.RedvoxPacketM) -> str:
        """
        Add (or update an existing session if key exists) a dynamic session with length of 1 day using a single packet

        :param packet: packet to read data from
        :return: the key to the new or updated dynamic session
        """
        data = smu.get_dynamic_data(packet)
        start_dt = dtu.datetime_from_epoch_microseconds_utc(packet.timing_information.packet_start_mach_timestamp)
        day_start_dt = dtu.datetime(start_dt.year, start_dt.month, start_dt.day)
        day_end_ts = int(dtu.datetime_to_epoch_microseconds_utc(day_start_dt + dtu.timedelta(days=1)))
        day_start_ts = int(dtu.datetime_to_epoch_microseconds_utc(day_start_dt))
        session_key = (
            f"{packet.station_information.id}:{packet.station_information.uuid}:"
            f"{packet.timing_information.app_start_mach_timestamp}"
        )
        key = f"{session_key}:{day_start_ts}:{day_end_ts}"
        hourly_key = self.add_dynamic_hour(data, packet.timing_information.packet_start_mach_timestamp, session_key)
        if key in self.dynamic_sessions.keys():
            self._update_dynamic_session(key, data, [hourly_key])
        else:
            self.dynamic_sessions[key] = cloud_sm.DynamicSession(
                1,
                smu.add_location_data(data["location"]),
                smu.add_to_stats(data["battery"]),
                smu.add_to_stats(data["temperature"]),
                session_key,
                day_start_ts,
                day_end_ts,
                DAILY_SESSION_NAME,
                [hourly_key],
            )
        return key

    def _update_dynamic_session(self, key: str, data: Dict, sub: List[str]):
        """
        update a dynamic session with a given key.

        :param key: key to the dynamic session
        :param data: dictionary of data to add
        :param sub: the list of keys that the dynamic session is linked to
        """
        if key not in self.dynamic_sessions.keys():
            self._errors.append(f"Attempted to update non-existent key: {key}.")
        else:
            dyn_sess = self.dynamic_sessions[key]
            dyn_sess.n_pkts += 1
            dyn_sess.location = smu.add_location_data(data["location"], dyn_sess.location)
            dyn_sess.battery = smu.add_to_stats(data["battery"], dyn_sess.battery)
            dyn_sess.temperature = smu.add_to_stats(data["temperature"], dyn_sess.temperature)
            for s in sub:
                if s in self.dynamic_sessions.keys():
                    self._update_dynamic_session(s, data, self.dynamic_sessions[s].sub)

    def sdk_version(self) -> str:
        """
        :return: sdk version used to create the SessionModel
        """
        return self._sdk_version

    def num_sensors(self) -> int:
        """
        :return: number of sensors in the Session
        """
        return len(self.cloud_session.sensors)

    def get_sensor_names(self) -> List[str]:
        """
        :return: number of sensors in the Session
        """
        return [n.name for n in self.cloud_session.sensors]

    def get_sensor(self, name: str, desc: Optional[str] = None) -> Optional[cloud_sm.Sensor]:
        """
        :param name: name of the sensor to get
        :param desc: Optional description of the sensor to get.  If None, will get the first sensor that
                        matches the name given.  Default None.
        :return: the first sensor that matches the name and description given or None if sensor was not found
        """
        for s in self.cloud_session.sensors:
            if s.name == name:
                if desc is None or s.description == desc:
                    return s
        return None

    def audio_sample_rate_nominal_hz(self) -> float:
        """
        :return: number of sensors in the Session
        """
        for n in self.cloud_session.sensors:
            if n.name == "audio":
                return n.sample_rate_stats.welford.mean

    def get_daily_dynamic_sessions(self) -> List[cloud_sm.DynamicSession]:
        """
        :return: all day-long dynamic sessions in the Session
        """
        return [n for n in self.dynamic_sessions.values() if n.dur == DAILY_SESSION_NAME]

    def get_hourly_dynamic_sessions(self) -> List[cloud_sm.DynamicSession]:
        """
        :return: all hour-long dynamic sessions in the Session
        """
        return [n for n in self.dynamic_sessions.values() if n.dur == HOURLY_SESSION_NAME]

    def print_errors(self):
        """
        print all errors encountered by the SessionModel
        """
        self._errors.print()


class LocalSessionModels:
    """
    SDK version of SessionModelsResp from the cloud API
    """

    def __init__(self):
        self.sessions: Dict[str, SessionModel] = {}

    def add_packet(self, packet: api_m.RedvoxPacketM) -> str:
        """
        add a packet to one of the models, or make a new one

        :param packet: packet of data to add.
        :return: key of new or updated packet
        """
        key = (
            f"{packet.station_information.id}:{packet.station_information.uuid}:"
            f"{int(packet.timing_information.app_start_mach_timestamp)}"
        )
        if key not in self.sessions.keys():
            self.sessions[key] = SessionModel.create_from_packet(packet)
        else:
            self.sessions[key].add_data_from_packet(packet)
        return self.sessions[key].cloud_session.session_key()

    def get_session(self, key: str) -> Optional[SessionModel]:
        """
        :param key: key of SessionModel to get
        :return: SessionModel that matches the given key or None
        """
        if key in self.sessions.keys():
            return self.sessions[key]
        return None
